unit KeyboardPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, stdctrls, inifiles;
const KEYSNUMBER = 31;
type
  TKeyboardPanel = class(TWinControl)
  private
    OriginalWidth, OriginalHeight : real;
    TheKeys : array[1..KEYSNUMBER] of String;
    ButtonsWidth, ButtonsHeight : integer;
    Selected : integer;
    IlFileIni : TIniFile;
    procedure ChangeKey(index : Integer ; S : String);
    function GetKey(index : Integer): String;
    private FOnKeyPress : TKeyPressEvent;
    procedure ButtonDragOver(Sender, Source: TObject; X, Y: Integer;
         State: TDragState; var Accept: Boolean);
    procedure ButtonDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure ReadIni;
  protected
    { Protected declarations }
  public
    Keyboard : array[1..KEYSNUMBER] of TButton;
    Callback : TNotifyEvent;
    function GetButtonIndex(sc : String) : integer;
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Repaint; override;
    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure SetButtonSelected(Sender: TObject);
    procedure SelectChar(sc : String);
    function GetCharSelected : String;
    procedure SaveIni;
  published
      property OnKeyboardClick: TNotifyEvent read Callback write Callback;
      property OnKeyPress;
  end;

var Chars : array[1..KEYSNUMBER] of String = (
             'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P',
             'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ';',
             'Z', 'X', 'C', 'V', 'B', 'N', 'M', ',', '.', '-','SPACE'
    );
procedure Register;

implementation


constructor TKeyboardPanel.Create(AOwner : TComponent);

  procedure CreateButtons;
  var
      i : integer;
  begin
    for i := 1 to KEYSNUMBER do begin
      Keyboard[i] := TButton.Create(self);
      Keyboard[i].OnKeyPress := OnKeyPress;
      TheKeys[i] := Chars[i];
      with Keyboard[i] do begin
        OnClick := SetButtonSelected;
        Caption := TheKeys[i];
        Visible := True;
        Parent := self;
        DragMode := dmAutomatic;
        OnDragDrop := ButtonDragDrop;
        OnDragOver := ButtonDragOver;
      end;
    end;
    Selected := 1;
    Repaint;
  end; {CreateButtons}

begin
  inherited Create(AOwner);
  OriginalWidth := Width;
  OriginalHeight := Height;
  CreateButtons;
  ReadIni;
end; {TKeyboardPanel.Create}

procedure TKeyboardPanel.Repaint;
var ButtonsLeftMargin : array[1..3] of integer;
    ButtonsTopMargin : integer;
    i,j:integer;
begin
  inherited Repaint;
  ButtonsLeftMargin[1] := Round(Width/20);
  ButtonsTopMargin := Round(Height/30);
  ButtonsWidth := Round((Width-2*ButtonsLeftMargin[1])/11);
  ButtonsHeight := Round((Height-2*ButtonsTopMargin)/4);
  ButtonsLeftMargin[2] := ButtonsLeftMargin[1]+Round(ButtonsWidth/2);
  ButtonsLeftMargin[3] := ButtonsLeftMargin[2]+Round(ButtonsWidth/2);
  for i := 1 to KEYSNUMBER-1 do begin
    with Keyboard[i] do begin
      Width := ButtonsWidth;
      Height := ButtonsHeight;
      Left := ButtonsLeftMargin[((i-1) div 10)+1] + ButtonsWidth*((i-1) mod 10);
      Top := ButtonsTopMargin + ((i-1) div 10)*ButtonsHeight;
      if i = Selected then begin
        Font.Size := round(sqrt(ButtonsHeight*ButtonsWidth)/2);
        Font.Style := [fsBold];
      end else begin
        Font.Size := round(sqrt(ButtonsHeight*ButtonsWidth)/2)-8;
        Font.Style := [];
      end;
    end;
  end;
  with Keyboard[KEYSNUMBER] do begin
    Width := self.width-ButtonsWidth*2-ButtonsLeftMargin[3]*2;
    Height := ButtonsHeight-10;
    Left := ButtonsLeftMargin[3] + ButtonsWidth*2;
    Top := ButtonsTopMargin + 3*ButtonsHeight;
    if KEYSNUMBER = Selected then begin
      Font.Size := round(sqrt(ButtonsHeight*ButtonsWidth)/2);
      Font.Style := [fsBold];
    end else begin
      Font.Size := round(sqrt(ButtonsHeight*ButtonsWidth)/2)-8;
      Font.Style := [];
    end;
  end;
end; {TKeyboardPanel.Repaint}

destructor TKeyboardPanel.Destroy;
var i :integer;
begin
  for i := 1 to KEYSNUMBER do
    Keyboard[i].Free;
  inherited Destroy;
end; {TKeyboardPanel.Destroy}

procedure TKeyboardPanel.WMSize(var Message: TWMSize);
begin
  Repaint;
end;

function TKeyboardPanel.GetKey(index : Integer): String;
begin
  result := TheKeys[index];
end;

procedure TKeyboardPanel.ChangeKey(index : Integer;S : String);
begin
  TheKeys[index] :=  S;
  Keyboard[index].Caption :=  S;
end;

procedure TKeyBoardPanel.SetButtonSelected(Sender: TObject);

  procedure ToggleSelected(Old, New : integer);
  begin
    Keyboard[Old].Font.Size := round(sqrt(ButtonsHeight*ButtonsWidth)/2)-8;
    Keyboard[Old].Font.Style := [];
    KeyBoard[New].Font.Size := round(sqrt(ButtonsHeight*ButtonsWidth)/2);
    Keyboard[New].Font.Style := [fsBold];
  end; { ToggleSelected }

var i: integer;
begin
  for i := 1 to KEYSNUMBER do
    if Keyboard[i] = Sender then begin
      ToggleSelected(Selected,i);
      Selected := i;
      if Assigned(Callback) then Callback(Sender);
        exit
    end;
end; {TKeyboardPanel.SetButtonSelected}

function TKeyBoardPanel.GetButtonIndex(sc : String) : integer;
var i: integer;
begin
  for i := 1 to KEYSNUMBER do
    if TheKeys[i] = sc then begin
      result := i;
      exit;
    end;
  result := 0;
end; { TKeyboardPanel.GetButtonIndex }

procedure TKeyBoardPanel.SelectChar(sc : String);
begin
  SetButtonSelected(Keyboard[GetButtonIndex(sc)]);
end; { TKeyBoardPanel.SelectChar }

function TKeyBoardPanel.GetCharSelected :String;
begin
  result := TheKeys[Selected];
end;

procedure TKeyBoardPanel.ButtonDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := Source is TButton;
end;

procedure TKeyboardPanel.ButtonDragDrop(Sender, Source: TObject; X, Y: Integer);
var Tempcaption, TempKey : String;
    From, Tob, i : integer;
begin
  for i := 1 to KEYSNUMBER do begin
    if (Keyboard[i] = Sender)  then tob := i;
    if (Keyboard[i] = Source) then from := i;
  end;
  Tempcaption := (Sender As TButton).Caption;
  (Sender As TButton).Caption := (Source As TButton).Caption;
  (Source As TButton).Caption := TempCaption;
  TempKey := TheKeys[from];
  TheKeys[From] := TheKeys[Tob];
  TheKeys[Tob] := tempKey;
end; {TKeyboardPanel.ButtonDragDrop}

procedure TKeyBoardPanel.SaveIni;
var i : integer;
    Keyb :String;
begin
  for i := 1 to KEYSNUMBER do begin
    Keyb := Concat('k',IntToStr(i));
    IlFileIni.WriteString('Keyboard',Keyb,TheKeys[i]);
  end;
  IlFileIni.Free;
  Halt;
end; {TKeyBoardPanel.SaveIni}

procedure TKeyBoardPanel.ReadIni;
var IniFile :String;
    WindowString : String;
    Windows : PChar;
    i : Integer;
    KeybStr : String;
begin
  SetLength(WindowString,MAX_PATH+1);
  i := GetWindowsDirectory(PChar(WindowString),MAX_PATH);
  SetLength(WindowString,i);
  IniFile := WindowString+'\keybpan.ini';
  ILFileIni := TIniFile.Create(IniFile);
  for i := 1 to KEYSNUMBER do begin
    KeybStr :=Concat('k',IntToStr(i));
    TheKeys[i]:= IlFileIni.ReadString('Keyboard',KeybStr,TheKeys[i]);
    Keyboard[i].Caption := TheKeys[i];
  end;
end; {TKeyboardPanel.ReadIni}

procedure Register;
begin
  RegisterComponents('Samples', [TKeyboardPanel]);
end;

end.
